@@ -1,16 +1,17 @@
-import traceback 
+import traceback
 from functools import update_wrapper
 import click
 import logging
 
-logger = logging.getLogger('app')
+logger = logging.getLogger("app")
+
 
 def log_activity(runfunc):
     """
-    This is a decorator that wraps around a click command. 
-    
+    This is a decorator that wraps around a click command.
+
     This decorator captures run metadata, initiates logging,
-    and handles exceptions during execution of the command. 
+    and handles exceptions during execution of the command.
 
     Example::
 
@@ -18,36 +19,36 @@ def log_activity(runfunc):
         @log_activity
         @click.pass_obj
         def _compare(config):
-          "Compares the data" 
+          "Compares the data"
            ....
-    
+
     """
-    @click.pass_context 
+
+    @click.pass_context
     def logged_run(ctx, *args, **kwargs):
         """
         Make note of what is going in and out...
         """
 
         config = ctx.obj
-        config.start_run(ctx.command.name) 
+        config.start_run(ctx.command.name)
         try:
             try:
-                config.state['command']['name'] = ctx.command_path
-                config.state['command']['description'] = ctx.command.help
+                config.state["command"]["name"] = ctx.command_path
+                config.state["command"]["description"] = ctx.command.help
             except:
-                traceback.print_exc() 
+                traceback.print_exc()
                 pass
 
             response = ctx.invoke(runfunc, *args, **kwargs)
-            config.set_status('success')
+            config.set_status("success")
             config.end_run()
-            return response 
-        except Exception as e: 
+            return response
+        except Exception as e:
             config.set_message(str(e))
-            config.set_status('failure')
+            config.set_status("failure")
             logger.exception("Failure in execution")
             config.end_run()
-            print("Command failed. Please see the log") 
+            print("Command failed. Please see the log")
 
     return update_wrapper(logged_run, runfunc)
-    
