@@ -8,7 +8,8 @@ __all__ = [
     "AnomaliesBase",
     "SyntheticDataGeneratorBase",
     "DataObserverBase",
-    "ChangePointDetectorBase"
+    "ChangePointDetectorBase",
+    "TimeSeriesForecasterBase",
 ]
 
 from .fileops import *
@@ -19,6 +20,7 @@ from .feature_compute import *
 from .metrics import *
 from .anomalies import *
 from .changepoints import *
+from .timeseries_forecaster import *
 from .synthetic_data_generator import *
 from .observability import *
 from .data_quality import *
