@@ -18,10 +18,12 @@ from matplotlib.patches import Rectangle
 
 from enrichsdk.contrib.lib.transforms import note
 from enrichsdk.contrib.lib.assets import profilespec
-from enrichsdk.contrib.lib.assets import ScribbleChangePointDetector
+from enrichsdk.contrib.lib.assets import BaseChangePointDetectorModel
 from enrichsdk.datasets import TransformDoodle
 from enrichsdk.utils import SafeEncoder
 
+from dateutil import parser as dateparser
+
 logger = logging.getLogger("app")
 
 class ChangePointDetectorBase(Compute):
@@ -47,9 +49,9 @@ class ChangePointDetectorBase(Compute):
         }
 
         self.cautions = {
-            "low": {"color": "green", "desc": "MINIMAL"},
-            "medium": {"color": "gold", "desc": "LOW to MODERATE"},
-            "high": {"color": "red", "desc": "HIGH to EXTREME"},
+            "low": {"color": "green", "desc": "MINIMAL", "rec": "In a BUSINESS-AS-USUAL regime now."},
+            "medium": {"color": "gold", "desc": "LOW to MODERATE", "rec": "Expect LOW to MODERATE swings in this regime."},
+            "high": {"color": "red", "desc": "HIGH to EXTREME", "rec": "Stay alert for HIGH to EXTREME swings in this regime."},
         }
 
         self.epoch = time.time()    #for output path
@@ -175,7 +177,7 @@ class ChangePointDetectorBase(Compute):
 
     def run_changepoint_detector(self, df, name):
         # init the change point detector object
-        cpd_obj = ScribbleChangePointDetector(df)
+        cpd_obj = BaseChangePointDetectorModel(df)
 
         # run the change point detection algorithm
         changepoints = cpd_obj.detect_changepoints(method="hybrid")
@@ -215,19 +217,27 @@ class ChangePointDetectorBase(Compute):
         lfactor_EXP = y_proj/y_mean
 
         # observed lift factor
-        lfactor_OBS = data[end]/y_proj
+        lfactor_OBS = data[end]/y_mean
 
         # impact
         if lfactor_EXP < 1 and lfactor_OBS < 1:
             impact = (lfactor_EXP) / (lfactor_OBS)
+        elif lfactor_EXP == 1:
+            impact = 0   # If the expected lift factor is 1 i.e there is no change before and after cp, impact is 0
         else:
             impact = (lfactor_OBS-1) / (lfactor_EXP-1)
 
+        direction = lfactor_OBS / lfactor_EXP
+        if direction < 1 and impact != 0 :
+            impact = 1 / impact
+
         return abs(impact)
 
 
     def compute_regimes(self, indicator_ts, observation_ts, changepoints):
         short_regime_range = 7
+        damping = 0.1
+        max_cap_factor = 1.5
 
         dates = observation_ts.index.values
         indicator_ts = indicator_ts.values
@@ -253,31 +263,44 @@ class ChangePointDetectorBase(Compute):
             # compute the impact factor at the change point for current regime
             impact_factor = self.compute_impact_factor(indicator_ts, start, end)
 
+            # cmopute the caution indicator
+            caution = self.compute_caution(impact_factor)
+
             # if very short current regime use prev two regimes
             if end-start <= short_regime_range:
                 start = prev_start
                 ts_values = observation_ts[start:end]
 
+            # get the ts values over last 30 day time period to account for month-end highs
+            ts_values_month = observation_ts[max(0,end-30):end]
+
+            # get prev, next regime dates
+            next_regime_dates = dates[changepoint_indxs[i+1]:changepoint_indxs[i+2]]
+            prev_regime_dates = dates[start:end]
+            prev_regime_len   = len(prev_regime_dates)-1
+
+
             # compute potential max ts value in the next regime
             if impact_factor >= 1:
                 m1 = max(ts_values)
-                m2 = min(ts_values_proj * math.sqrt(impact_factor), max(ts_values)*1.2)
-                m3 = min(np.mean(ts_values) * math.sqrt(impact_factor), max(ts_values)*1.2)
+                m2 = min(ts_values_proj*math.sqrt(impact_factor), max(ts_values)*(impact_factor**damping))
+                m3 = min(np.median(ts_values)*math.sqrt(impact_factor), max(ts_values)*(impact_factor**damping))
                 max_ts_values = [round(x) for x in sorted([m1, m2, m3], reverse=True)]
+                if caution == 'low':
+                    max_cap = np.median(ts_values) * max_cap_factor
+                    max_ts_values = [round(min(x, max_cap)) for x in max_ts_values]
+                # we have to consider the last month high
+                max_ts_values = sorted(max_ts_values + [round(max(ts_values_month))], reverse=True)[0:3]
 
                 min_ts_values = [round(x) for x in sorted([min(ts_values), min(ts_values), min(ts_values)])]
             else:
                 max_ts_values = [round(x) for x in sorted([max(ts_values), max(ts_values), max(ts_values)])]
 
-                m1 = min(ts_values)
+                m1 = min(ts_values_month) #min(ts_values)
                 m2 = ts_values_proj * math.sqrt(impact_factor)
                 m3 = np.mean(ts_values) * math.sqrt(impact_factor)
                 min_ts_values = [round(x) for x in sorted([m1, m2, m3])]
 
-            # get prev, next regime dates
-            next_regime_dates = dates[changepoint_indxs[i+1]:changepoint_indxs[i+2]]
-            prev_regime_dates = dates[start:end]
-
             # generate the context string
             rd = {
                 "start_date": next_regime_dates[0],
@@ -287,33 +310,49 @@ class ChangePointDetectorBase(Compute):
                 "impact_factor": impact_factor,
                 "min_value": min_ts_values,
                 "max_value": max_ts_values,
+                "prev_max_value": max(ts_values),
             }
             context = self.generate_regime_context(rd)
             rd['context'] = context
 
             regime_data.append(rd)
 
+        # add the special case of final regime
+        context = self.generate_regime_context(rd, final_rec=True)
+        rd['context'] = context
+        regime_data[-1] = rd
+
         return regime_data
 
     def compute_caution(self, impact_factor):
         caution = 'low'
-        if impact_factor > 3 and impact_factor <= 5:
+        if impact_factor > 4 and impact_factor <= 6:
             caution = "medium"
-        if impact_factor > 5:
+        if impact_factor > 6:
             caution = "high"
         return caution
 
 
-    def generate_regime_context(self, rd):
+    def generate_regime_context(self, rd, final_rec=False):
+
+        def millify(n):
+            n = float(n)
+            millnames = ['','K','M','B']
+            millidx = max(0,min(len(millnames)-1,
+                                int(math.floor(0 if n == 0 else math.log10(abs(n))/3))))
+            return str(round(n / 10**(3 * millidx), 2)) + str(millnames[millidx])
+
         impact_factor = rd['impact_factor']
         caution = self.compute_caution(impact_factor)
 
         if caution == 'low':
-            high_str = f"around ${rd['max_value'][0]}"
+            high_str = f"around ${millify(rd['max_value'][0])}"
         if caution == 'medium':
-            high_str = f"between ${rd['max_value'][1]} and ${rd['max_value'][0]}"
+            high_str = f"between ${millify(rd['max_value'][1])} and ${millify(rd['max_value'][0])}"
         if caution == 'high':
-            high_str = f"between ${rd['max_value'][2]} and ${rd['max_value'][1]} in the near future"
+            high_str = f"between ${millify(rd['max_value'][2])} and ${millify(rd['max_value'][1])} in the near future"
+
+        multiple_str = round(rd['max_value'][0] / rd['prev_max_value'], 2)
 
         regime_start_date       = pd.to_datetime(str(rd['start_date'])).strftime("%-d %B, %Y")
         prev_regime_start_date  = pd.to_datetime(str(rd['prev_regime_start_date'])).strftime("%-d %B, %Y")
@@ -321,7 +360,11 @@ class ChangePointDetectorBase(Compute):
 
         context = [f"{regime_start_date}: Regime change detected."] + \
                     [f"Expecting {self.cautions[caution]['desc']} change in spikiness from previous regime spanning {prev_regime_start_date} to {prev_regime_end_date}."] + \
-                    [f"Recommendation: Expect highs {high_str}."]
+                    [f"Recommendation 1: Expect highs around {multiple_str}x of previous regime high."] + \
+                    [f"Recommendation 2: Expect highs {high_str}."]
+
+        if final_rec:
+            context[1] = f"{self.cautions[caution]['rec']}"
 
         return " ".join(context)
 
@@ -393,10 +436,11 @@ class ChangePointDetectorBase(Compute):
             max_ts_value = max_ts_value if max(rd['max_value'])<max_ts_value else max(rd['max_value'])
             sgn *= -1
 
-        ax.set_ylim(top=max_ts_value)
+        ax.set_ylim(bottom=0, top=max_ts_value)
         ax.set_ylabel('Value')
         ax.set_title('Projected Regime Bands')
 
+        fig.autofmt_xdate()
         return plt
 
 
@@ -501,7 +545,7 @@ class ChangePointDetectorBase(Compute):
                     {
                         "type": "file",
                         "nature": "input",
-                        "objects": [paths],
+                        "objects": paths,
                     },
                 ],
             }
@@ -511,13 +555,16 @@ class ChangePointDetectorBase(Compute):
 
         return df
 
-    def load_dataset(self, spec, name, source):
+    def load_dataset(self, spec, name, source, datewindow):
         msg = ""
 
         source_id       = source['source_id']
         dataset         = source['dataset']
         source_version  = source.get('source_version', 'v1')
 
+        start_date      = datewindow['start_date']
+        end_date        = datewindow['end_date']
+
         if source_id == "custom":
             # we have a custom defined method in the derived class to generate the dataset
             if not hasattr(self, dataset):
@@ -526,33 +573,23 @@ class ChangePointDetectorBase(Compute):
                     extra={"transform": self.name, "data": json.dumps(source, indent=4)}
                 )
                 return None
+
             handler = getattr(self, dataset)
             params = source.get("params", {})
-            df = handler(params)
+            df = handler(start_date, end_date, params)
+            if df is None:
+                logger.exception(
+                    f"Couldn't load dataset for {spec['name']}",
+                    extra={"transform": self.name, "data": json.dumps(source, indent=4)}
+                )
+                return None
+
             msg += f"Loaded using custom method: {dataset}" + "\n"
         else:
             # we are using the SDK to get the dataset
             datacred = self.args['datacred']
             doodle = TransformDoodle(self, self.state, datacred)
 
-            # defaults
-            start_date = self.args['run_date'] + timedelta(days=-7)
-            end_date = self.args['run_date']
-
-            # check if we need to get specific start and end times for the dataset
-            timewindow = source.get("timewindow")
-            if timewindow != None:
-                if not hasattr(self, timewindow['method']):
-                    logger.exception(
-                        f"No timewindow handler for: {dataset}",
-                        extra={"transform": self.name, "data": json.dumps(source, indent=4)}
-                    )
-                else:
-                    handler = getattr(self, timewindow['method'])
-                    params = timewindow.get("params", {})
-                    start_date, end_date = handler(params)
-                    msg += f"Time window: Start date={start_date}, End date={end_date}" + "\n"
-
             d_source, paths = doodle.get_source_paths(start=start_date,
                                                     end=end_date,
                                                     name=dataset,
@@ -672,6 +709,8 @@ class ChangePointDetectorBase(Compute):
                         extra={"transform": self.name,
                                 "data": msg})
 
+
+
     def db_store_result(self, spec, result):
         name    = spec['name']
         config  = spec['config']
@@ -729,6 +768,49 @@ class ChangePointDetectorBase(Compute):
                              'transform': self.name
                          })
 
+
+    def get_datewindow(self, source, spec):
+        datewindow = {}
+        default_delta = 180
+        
+        start_date_str = source.get('start_date')
+        end_date_str = source.get('end_date')
+
+        if not end_date_str:
+            logger.debug(f"end date not specified in {spec['name']}. Using default as yesterday's date.")
+            end  = self.args['run_date'] # yesterday
+            end_date_str = end.isoformat()
+
+        if not start_date_str:
+            logger.debug(f"start date not specified in {spec['name']}. Using default as 180 days prior to end date. ")
+            start = dateparser.parse(end_date_str).date() + timedelta(days=-default_delta)
+            start_date_str = start.isoformat()
+
+       # If start and end dates are specified, check if date is parseable
+        try:
+            start_date = dateparser.parse(start_date_str).date()
+            end_date = dateparser.parse(end_date_str).date()
+        except:
+            logger.exception(
+                        f"Invalid start date or end date. Skipping the spec {spec['name']}.",
+                        extra={"transform": self.name, "data": json.dumps(spec, indent=4)}
+                    )
+            datewindow = None # be explicit
+            return datewindow
+
+        if start_date > end_date:
+            logger.exception(
+                        f"Start date greater than end date. Skipping the spec {spec['name']}.",
+                        extra={"transform": self.name, "data": json.dumps(spec, indent=4)}
+                    )
+            datewindow = None # be explicit
+            return datewindow
+
+        datewindow['start_date'] = start_date
+        datewindow['end_date'] = end_date
+
+        return datewindow
+
     ###########################################
     # Expectation Testing Functions
     ###########################################
@@ -808,17 +890,27 @@ class ChangePointDetectorBase(Compute):
                     )
                     do_process_spec = False
                     break
+            # expections have occured. continue to next spec
             if do_process_spec == False:
                 continue
 
-            # frist, load the indicator dataset
+            # get time_window for indicator and observation data set
+            datewindow = self.get_datewindow(source, spec)
+            if datewindow is None :
+                do_process_spec = False
+                continue
+
+            # first, load the indicator dataset
             data = {}
-            data['indicator'] = self.load_dataset(spec, 'indicator', source['indicator'])
+            data['indicator'] = self.load_dataset(spec, 'indicator', source['indicator'], datewindow)
+            if data['indicator'] is None:
+                do_process_spec = False
+                continue
 
             # then, load all the observations datasets
             data['observations'] = {}
             for ob_dataset, dataset in source['observations'].items():
-                data['observations'][ob_dataset] = self.load_dataset(spec, ob_dataset, dataset)
+                data['observations'][ob_dataset] = self.load_dataset(spec, ob_dataset, dataset, datewindow)
 
             # then, process it
             result = self.process_spec(spec, data)
