@@ -10,6 +10,7 @@ import tempfile
 import sqlite3
 import jsonschema
 from functools import partial
+from collections import defaultdict
 
 import numpy as np
 import pandas as pd
@@ -133,14 +134,38 @@ class SQLExport(Sink):
         """
         Execute the export specification.
         """
+
+        exports =  self.args["exports"]
+
+
+        # Look at messages sent from transforms and collect the
+        # frames.
+        extra = defaultdict(list)
+        while True:
+            msg = state.msgpop(self.name)
+            if msg is None:
+                break
+            if not isinstance(msg, dict):
+                continue
+            data = msg.get('data', None)
+            if ((isinstance(data, dict)) and
+                ('frames' in data)):
+                name = data.get('name', 'default')
+                frames = data['frames']
+                extra[name].extend(frames)
+
         # Sanity check...
-        for e in self.args["exports"]:
+        for e in exports:
+
+            name = e.get('name', 'default')
 
             # Collect all the frames data
             missing = []
             invalid = []
             frames = {}
-            for f in e["frames"]:
+
+            allframes = e["frames"] + extra.get(name, [])
+            for f in allframes:
                 detail = state.get_frame(f)
                 if detail is None:
                     missing.append(f)
