@@ -1749,19 +1749,18 @@ class Transform(Node):
                 # Nothing to parse
                 return detail.get("description", "")
 
+            matcher = detail['matcher']
+
             # Now parse and apply to generate
-            variables = detail.get("variables", [])
-            context = {}
-            for v in variables:
-                try:
-                    context[v] = matcher.group(v)
-                except:
-                    context[v] = "error"
 
             try:
-                return generator % context
+                context = matcher.groupdict()
+                if callable(generator):
+                    return generator(context)
+                else:
+                    return generator % context
             except:
-                logger.error("Error in configuration", extra={"transform": self.name})
+                logger.error("Failed to generate documentation", extra={"transform": self.name})
                 pass
 
         return ""
