@@ -31,6 +31,7 @@ import getpass
 import distro
 from datetime import datetime
 
+from ..lib.exceptions import *
 
 class EnrichStateBase(object):
     """
@@ -56,6 +57,7 @@ class EnrichStateBase(object):
         self.performance_summary = []
         self.dataset_accesses = []
         self.frames = {}
+        self.kpi = {}
         self.messages = {}
         self.transforms = {}
         self.transform_overrides = {}
@@ -78,7 +80,7 @@ class EnrichStateBase(object):
                 "release": platform.release(),
                 "processor": platform.processor(),
                 "python": platform.python_version(),
-                "distribution": distro.linux_distribution(),
+                "distribution": (distro.id().title(), distro.version(), distro.name())
             },
             "user": user,
         }
@@ -159,6 +161,21 @@ class EnrichStateBase(object):
         """
         pass
 
+    def update_kpi(self, name, value):
+        """
+        Add a KPI to the pipeline. We can
+
+        Args:
+            name (str): Name of the KPI
+            value (obj): Serializable value (int/str/float)
+        """
+        if not isinstance(name, str):
+            raise InvalidParameterValue(f"Invalid kpi name: {name}")
+        if not isinstance(value, (str, int, float, bool)):
+            raise InvalidParameterValue(f"Invalid kpi value: {value}")
+
+        self.kpi[name] = value
+
     def make_dataset_access_note(self, dataset, metadata, nature="read-write"):
         self.dataset_accesses.append({
             'timestamp': datetime.now(),
