@@ -15,6 +15,8 @@ def get_lineage_of_query(engine, sql):
     result = LineageRunner(sql, verbose=True)
 
     database = engine.url.database
+    if database is None:
+        database = "<default>"
 
     # Get input tables..
     if len(result.source_tables) > 0:
@@ -122,6 +124,11 @@ def note(df, title):
         str: A formatted text to be used for logging
 
     """
+
+    # May be the parameters have been flipped
+    if isinstance(df, str):
+        df, title = title, df
+
     msg = title + "\n"
     msg += "--------" + "\n"
     msg += "Timestamp: " + str(datetime.now()) + "\n"
@@ -138,7 +145,7 @@ def note(df, title):
 ############################
 
 def get_month():
-    return datetime.utcnow().strftime("%Y-%m")
+    return datetime.now().strftime("%Y-%m")
 
 def get_yesterday():
     yesterday = date.today() + timedelta(days=-1)
@@ -150,3 +157,4 @@ def get_tomorrow():
 
 def get_today():
     return date.today().isoformat()
+
